import pygame
from pygame.locals import *
from pgu import isovid
import sys, os
import logging 
from pgu import gui
from map import *
import httplib;
from gfxcursor import *
from intro import *

SW,SH = 1024, 768 
TW,TH = 64,64

#****************************************************************************
#
#****************************************************************************
class IsoScreen:
  def __init__(self, screen):
    self.TW = TW;
    self.TH = TH;
    self.setup(screen);
    self.show_game_start();
    self.run();


#****************************************************************************
#
#****************************************************************************
  def setup(self, screen):
    self.view = isovid.Isovid();
    self.clock = pygame.time.Clock();
    self.frame_counter = 0;

    self.view.screen = screen; 
    self.active_combat = 0;
    pygame.font.init();
    self.view.font = pygame.font.SysFont('helvetica', 20);
 
    self.map = Map(self);
    self.set_status_text("Game started. Your objective is to collect coal and water, and eliminate your enemies.");
    self.view.tga_load_tiles(os.path.join('data', 'isotiles.tga'),(TW,TH));
    self.view.tga_load_level(os.path.join('data', 'isolevel.tga'),1);
    self.load_graphics();
    self.highscores = 0;
    self.app = gui.App();
    x, y = self.view.tile_to_view((25,25));
    self.view.view.x, self.view.view.y = x - (SW/2), y - (SH/2);
    image = pygame.image.load(os.path.join('data', 'cursor.png'));
    self.cursor = GfxCursor(screen, image, (16, 16))


#****************************************************************************
#
#****************************************************************************
  def combat_attack(self, attacker, enemy): 
 
    logging.info("You are attacking an enemy");
    self.map.player_water -= 1;
    self.map.player_coal -= 1;
    self.active_combat = 1;
    self.attacker = attacker;
    self.enemy = enemy;
    container = gui.Container(align=-1, valign=-1);
    container.add(gui.Image(self.graphics['background']), 50,10);
    self.message_label = gui.Label("Combat:  You have chosen to attack an enemy. Choose your insult wisely!");
    container.add(self.message_label, 200, 90);
    i = 0;
    for insult in attacker.insults:
      message_button = gui.Button(insult, width=600);
      container.add(message_button, 200, 110 + i * 25);
      message_button.connect(gui.CLICK, self.say_insult, message_button);

      i += 1;
    self.app.init(container); 

#****************************************************************************
#
#****************************************************************************
  def combat_defend(self, enemy, defender): 
    logging.info("Enemy attacking you!");
    self.active_combat = 1;
    self.defender = defender;
    self.enemy = enemy;
    container = gui.Container(align=-1, valign=-1);
    container.add(gui.Image(self.graphics['background']), 50,10);
    self.insult = random.choice(self.enemy.insults);
    self.message_label = gui.Label("Combat! Enemy insult: \"%s\"" % self.insult);
    container.add(self.message_label, 200, 25);

    self.message_label = gui.Label("Choose your comeback wisely!");
    container.add(self.message_label, 200, 65);

    i = 0;
    for insult in defender.comebacks:
      message_button = gui.Button(insult, width=600);
      container.add(message_button, 200, 100 + i * 25);
      message_button.connect(gui.CLICK, self.say_comeback, message_button);
      i += 1;
    self.app.init(container); 

#****************************************************************************
#
#****************************************************************************
  def say_insult(self, button): 
    insult = button.value.value;
    self.comeback = self.attacker.insultengine.get_comeback_from_insult(insult);
    if self.enemy.comebacks.count(self.comeback) != 0:
      logging.info("Enemy did have the correct comeback");
      result = self.attacker.attack(self.enemy, insult, self.comeback); 
    else:
      logging.info("Enemy did not have the correct comeback");
      self.comeback = random.choice(self.enemy.comebacks);
      result = self.attacker.attack(self.enemy, insult, self.comeback); 

    # Display GUI of results....
    container = gui.Container(align=-1, valign=-1);
    container.add(gui.Image(self.graphics['background']), 50,10);

    self.message_label = gui.Label("Your insult: \"%s\"" % insult);
    container.add(self.message_label, 200, 100);

    self.message_label = gui.Label("Enemy comback: \"%s\"" % self.comeback);
    container.add(self.message_label, 200, 130);

    self.map.moves = 0;


    if result == 0:
      message = "Combat result: You won the battle!";
      self.message_label = gui.Label(message);
      self.set_status_text(message);
      self.map.score += 10;
      self.map.enemy_list.remove(self.enemy);
      self.map.enemies -= 1;
      print self.map.enemies;
    else:
      message = "Combat result: You lost the battle!";
      self.message_label = gui.Label(message);
      self.set_status_text(message);

      self.map.score -= 20;
    container.add(self.message_label, 200, 180);



    message_button = gui.Button("OK", width=130);
    container.add(message_button, 450, 550 );
    message_button.connect(gui.CLICK, self.accept, message_button);

    self.app.init(container); 


#****************************************************************************
#
#****************************************************************************
  def say_comeback(self, button): 
    comeback = button.value.value;


    result = self.enemy.attack(self.defender, self.insult, comeback); 

    # Display GUI of results....
    container = gui.Container(align=-1, valign=-1);
    container.add(gui.Image(self.graphics['background']), 50,10);

    self.message_label = gui.Label("Enemy insult: \"%s\"" % self.insult);
    container.add(self.message_label, 200, 100);

    self.message_label = gui.Label("Your comback: \"%s\"" % comeback);
    container.add(self.message_label, 200, 130);


    if result == 1:
      message = "Combat result: You won the battle!";
      self.message_label = gui.Label(message);
      self.set_status_text(message);

      self.map.enemy_list.remove(self.enemy);
      self.map.enemies -= 1;
      self.map.score += 10;
      print self.map.enemies;
    else:
      message = "Combat result: You lost the battle!";
      self.message_label = gui.Label(message);
      self.set_status_text(message);
      self.map.score -= 20;
      self.map.player_water = 0;
      self.map.player_coal = 0;

    container.add(self.message_label, 200, 180);



    message_button = gui.Button("OK", width=130);
    container.add(message_button, 450, 550 );
    message_button.connect(gui.CLICK, self.accept, message_button);

    self.app.init(container); 


#****************************************************************************
#
#****************************************************************************
  def accept(self, button): 
    container = gui.Container(align=-1, valign=-1);
    self.app.init(container); 
    self.active_combat = 0;
    pygame.mixer.music.load(os.path.join('data', 'music1.ogg'));
    #music is looping.
    pygame.mixer.music.play(-1);

 
#****************************************************************************
#
#****************************************************************************
  def load_graphics(self): 
    self.graphics = {};
    player_vent = pygame.image.load(os.path.join('data', 'mech_vent.png'));
    for i in range(8):
      self.graphics.update({'player-vent-%r-0' % i: player_vent.subsurface(i*96,0,96,96)});
      self.graphics.update({'player-vent-%r-1' % i: player_vent.subsurface(i*96,96,96,96)});
      self.graphics.update({'player-vent-%r-2' % i: player_vent.subsurface(i*96,96*2,96,96)});
      self.graphics.update({'player-vent-%r-3' % i: player_vent.subsurface(i*96,96*3,96,96)});
    player_walk = pygame.image.load(os.path.join('data', 'mech_walk.png'));
    for i in range(8):
      self.graphics.update({'player-walk-%r-0' % i: player_walk.subsurface(i*96,0,96,96)});
      self.graphics.update({'player-walk-%r-1' % i: player_walk.subsurface(i*96,96,96,96)});
      self.graphics.update({'player-walk-%r-2' % i: player_walk.subsurface(i*96,96*2,96,96)});
      self.graphics.update({'player-walk-%r-3' % i: player_walk.subsurface(i*96,96*3,96,96)});
      self.graphics.update({'player-walk-%r-4' % i: player_walk.subsurface(i*96,96*4,96,96)});
      self.graphics.update({'player-walk-%r-5' % i: player_walk.subsurface(i*96,96*5,96,96)});
      self.graphics.update({'player-walk-%r-6' % i: player_walk.subsurface(i*96,96*6,96,96)});
      self.graphics.update({'player-walk-%r-7' % i: player_walk.subsurface(i*96,96*7,96,96)});
    player_collect = pygame.image.load(os.path.join('data', 'mech_collect.png'));
    for i in range(8):
      self.graphics.update({'player-collect-%r-0' % i: player_collect.subsurface(i*96,0,96,96)});
      self.graphics.update({'player-collect-%r-1' % i: player_collect.subsurface(i*96,96,96,96)});
      self.graphics.update({'player-collect-%r-2' % i: player_collect.subsurface(i*96,96*2,96,96)});
      self.graphics.update({'player-collect-%r-3' % i: player_collect.subsurface(i*96,96*3,96,96)});
    player_can = pygame.image.load(os.path.join('data', 'mech_cannon.png'));
    for i in range(8):
      self.graphics.update({'player-cannon-%r-0' % i: player_can.subsurface(i*96,0,96,96)});
      self.graphics.update({'player-cannon-%r-1' % i: player_can.subsurface(i*96,96,96,96)});
      self.graphics.update({'player-cannon-%r-2' % i: player_can.subsurface(i*96,96*2,96,96)});
      self.graphics.update({'player-cannon-%r-3' % i: player_can.subsurface(i*96,96*3,96,96)});
    infantry = pygame.image.load(os.path.join('data', 'infantry_walk.png'));
    for i in range(8):
      self.graphics.update({'infantry-%r-0' % i: infantry.subsurface(i*96,0,96,96)});
      self.graphics.update({'infantry-%r-1' % i: infantry.subsurface(i*96,96,96,96)});
      self.graphics.update({'infantry-%r-2' % i: infantry.subsurface(i*96,96*2,96,96)});
      self.graphics.update({'infantry-%r-3' % i: infantry.subsurface(i*96,96*3,96,96)});
      self.graphics.update({'infantry-%r-4' % i: infantry.subsurface(i*96,96*4,96,96)});
      self.graphics.update({'infantry-%r-5' % i: infantry.subsurface(i*96,96*5,96,96)});
      self.graphics.update({'infantry-%r-6' % i: infantry.subsurface(i*96,96*6,96,96)});
      self.graphics.update({'infantry-%r-7' % i: infantry.subsurface(i*96,96*7,96,96)});

    coal = pygame.image.load(os.path.join('data', 'coal.png'));
    self.graphics.update({'coal': coal});
    water = pygame.image.load(os.path.join('data', 'water.png'));
    self.graphics.update({'water': water});
    steam1 = pygame.image.load(os.path.join('data', 'steamR-1.png'));
    steam2 = pygame.image.load(os.path.join('data', 'steamR-2.png'));
    steam3 = pygame.image.load(os.path.join('data', 'steamR-5.png'));
    steam4 = pygame.image.load(os.path.join('data', 'steamR-6.png'));
    steam = [steam1, steam2, steam3, steam4]
    self.graphics.update({'steam': steam});
    panel = pygame.Surface((SW, 90));
    panel.fill((0,0,0));
    panel.set_alpha(158);
    panel.convert();
    self.graphics.update({'panel': panel});
    panel = pygame.Surface((900, 650));
    panel.fill((255,0,0));
    panel.set_alpha(120);
    panel.convert();
    self.graphics.update({'background': panel});

#****************************************************************************
#
#****************************************************************************
  def show_game_start(self): 
 
    container = gui.Container(align=-1, valign=-1);
    container.add(gui.Image(self.graphics['background']), 50,20);
    self.message_label = gui.Label("/****    Mission orders    *****/");

    container.add(self.message_label, 290, 150 );

    self.message_label2 = gui.Label("- You are ordered to kill all the enemies with your insults.");
    container.add(self.message_label2, 220, 220 );
    self.message_label3 = gui.Label("- Collect coal and water in order to fuel your steam-engine.");
    container.add(self.message_label3, 220, 260 );
    self.message_label2 = gui.Label("- Learn insults and comebacks in order to attack more effectively.");
    container.add(self.message_label2, 220, 300 );
    self.message_label2 = gui.Label("- Move your unit with the keypad or mouse, and look around with the right mouse button.");
    container.add(self.message_label2, 220, 340 );
    self.message_label2 = gui.Label("- The game is turn-based. Press Enter to end your turn.");
    container.add(self.message_label2, 220, 380 );

    self.message_label2 = gui.Label("- Good luck, you're gonna need it!");
    container.add(self.message_label2, 220, 420 );

    message_button = gui.Button("Yes my lord!", width=130);
    container.add(message_button, 450, 550 );
    message_button.connect(gui.CLICK, self.start_game, message_button);
    self.app.init(container); 

#****************************************************************************
#
#****************************************************************************
  def start_game(self, blah): 
    container = gui.Container(align=-1, valign=-1);
    self.app.init(container); 


#****************************************************************************
#
#****************************************************************************
  def show_score(self): 
 
    logging.info("The game is over!");
    self.highscores = 1;
    self.ptr = self.map.score;
    container = gui.Container(align=-1, valign=-1);
    container.add(gui.Image(self.graphics['background']), 100,100);
    self.message_label = gui.Label("Mission accomplished! You won the game, with the score %r." % self.map.score);

    container.add(self.message_label, 270, 200 );

    self.message_label2 = gui.Label("Please enter your name for the highscore:");

    container.add(self.message_label2, 270, 250 );

    self.input = gui.Input();
    container.add(self.input, 350, 280 );

    message_button = gui.Button("OK", width=130);
    container.add(message_button, 450, 550 );
    message_button.connect(gui.CLICK, self.submit, message_button);
    self.app.init(container); 

#****************************************************************************
#
#****************************************************************************
  def end(self, blah): 
    sys.exit(0);

#****************************************************************************
#
#****************************************************************************
  def submit(self, blah): 
    conn = httplib.HTTPConnection("www.openrts.org")
    import urllib;
    str = "/pyweek/score.php?a=%s&r=%r" % (urllib.quote(self.input.value),self.ptr);
    conn.request("GET", str);
    r1 = conn.getresponse();
    data1 = r1.read();
    conn.close();
    self.app.quit();
    print "Thank you for playing."
#****************************************************************************
#
#****************************************************************************
  def run(self): 
    
    self.view.paint(self.view.screen);
    pygame.display.flip();
    
    while 1:
      self.clock.tick(8);
      self.handle_events();
      self.view.loop();
      self.view.screen.fill((0,0,0));
      updates = self.view.update(self.view.screen);
      self.draw_units();
      self.draw_panel();
      self.app.paint(self.view.screen);
      self.cursor.show();
      pygame.display.flip();


#****************************************************************************
#
#****************************************************************************
  def animate(self): 
    for i in range(9):
      self.clock.tick(14);
      self.handle_events();
      self.view.loop();
      self.view.screen.fill((0,0,0));
      updates = self.view.update(self.view.screen);
      self.draw_units();
      self.draw_panel();
      self.app.paint(self.view.screen);
      pygame.display.flip();

#****************************************************************************
#
#****************************************************************************
  def draw_units(self):
    i = 8
    for water_pos in self.map.water_list:
      x,y = self.view.tile_to_screen(water_pos);
      self.view.screen.blit(self.graphics['water'], (x-TW/2, y-TH/2)); 

    for coal_pos in self.map.coal_list:
      x,y = self.view.tile_to_screen(coal_pos);
      self.view.screen.blit(self.graphics['coal'], (x-TW/2, y-TH/2)); 

    self.draw_player();

 

#****************************************************************************
#
#****************************************************************************
  def set_status_text(self, text): 
    self.status_text = text;

#****************************************************************************
#
#****************************************************************************
  def draw_player(self): 
    x, y = self.view.tile_to_screen(self.map.player_pos);
    dir = 5;
    x1, y1 = self.map.old_player_pos;
    x2, y2 = self.map.player_pos;
    if x1 == x2 and y1 == y2: dir = 5;
    if x1 + 1 == x2 and y1 == y2: dir = 7;
    if x1 - 1 == x2 and y1 == y2: dir = 3;
    if x1 - 1 == x2 and y1 - 1 == y2: dir = 2;
    if x1 + 1 == x2 and y1 + 1 == y2: dir = 6;
    if x1 + 1 == x2 and y1 - 1 == y2: dir = 0;
    if x1 == x2 and y1 - 1 == y2: dir = 5;
    if x1  == x2 and y1 - 1  == y2: dir = 1;
    if x1 - 1 == x2 and y1 + 1  == y2: dir = 4;
    self.frame_counter += 1;
    state = self.map.player_state;
    if state == "walk":
      frame = self.frame_counter % 8;
      dx, dy = self.map.player_offset; 
      self.map.player_offset = (int(dx + (-1*dx/7)), int(dy + (-1*dy/5)));
      logging.info("%r %r" % self.map.player_offset);
      self.view.screen.blit(self.graphics['player-%s-%r-%r' % (state, dir, frame)], (x-TW/2+dx-16, y-TH/2+dy-10));
    if state == "vent":
      frame = self.frame_counter % 4;
      self.view.screen.blit(self.graphics['player-%s-%r-%r' % (state, dir, frame)], (x-TW/2-16, y-TH/2-10));
    if state == "collect":
      frame = self.frame_counter % 4;
      self.view.screen.blit(self.graphics['player-%s-%r-%r' % (state, dir, frame)], (x-TW/2-16, y-TH/2-10));
    if state == "cannon":
      frame = self.frame_counter % 4;
      self.view.screen.blit(self.graphics['player-%s-%r-%r' % (state, dir, frame)], (x-TW/2-16, y-TH/2-10));
    frame = self.frame_counter % 8;
    for enemy in self.map.enemy_list:
      x, y = self.view.tile_to_screen((enemy.x, enemy.y));
      self.view.screen.blit(self.graphics['infantry-%r-%r' % (enemy.dir, frame)], (x-TW/2-16,y-TH/2-10)); 


#****************************************************************************
#
#****************************************************************************
  def draw_panel(self): 
    self.view.screen.blit(self.graphics['panel'], (0, SH - 90)); 
    img = self.view.font.render(self.status_text, 1, (255, 255, 0));
    self.view.screen.blit(img,(200, SH - 60));
    img = self.view.font.render("Current Turn: %r" % (self.map.turn), 1, (255, 0, 0));
    self.view.screen.blit(img,(20, SH - 90));
    img = self.view.font.render("Your Time Units: %r" % (self.map.moves), 1, (255, 0, 0));
    self.view.screen.blit(img,(20, SH - 70));
    img = self.view.font.render("Enemies: %r Score: %r" % (len(self.map.enemy_list), self.map.score), 1, (255, 0, 0));
    self.view.screen.blit(img,(20, SH - 50));
    img = self.view.font.render("Insults: %r Comebacks %r" % (len(self.map.user.insults) ,len(self.map.user.comebacks)), 1, (255, 0, 0));
    self.view.screen.blit(img,(20, SH - 30));

    for i in range(self.map.player_water):
      self.view.screen.blit(self.graphics['water'], (SW - 70 - i * 35, SH - 60)); 
    for i in range(self.map.player_coal):
      self.view.screen.blit(self.graphics['coal'], (SW - 70 - i * 35,  SH - 100)); 



#****************************************************************************
#
#****************************************************************************
  def handle_events(self):
    for e in pygame.event.get():
       self.cursor.update(e)
       if e.type is QUIT: sys.exit(0);

       if e.type is KEYDOWN and e.key == K_ESCAPE: sys.exit(0);

       if e.type is MOUSEBUTTONDOWN and e.button == 3:
         mouse_pos = pygame.mouse.get_pos(); 
         tile = self.view.screen_to_tile(mouse_pos);
         x, y = self.view.tile_to_view(tile);
         self.view.view.x, self.view.view.y = x - (SW/2), y - (SH/2);



       if e.type is MOUSEBUTTONDOWN and e.button == 1:
         mouse_pos = pygame.mouse.get_pos(); 
         mx, my = self.view.screen_to_tile(mouse_pos);
         px, py = self.map.player_pos;
 
         if (self.map.moves != 0 and (mx + 1 == px and my + 1 == py 
            or mx + 1 == px and my == py
            or mx + 1 == px and my - 1 == py
            or mx == px and my - 1 == py
            or mx == px and my + 1 == py
            or mx - 1 == px and my - 1 == py
            or mx - 1 == px and my == py
            or mx - 1 == px and my + 1 == py)):
           self.map.move_player_to((mx,my)); 
         else:
           self.set_status_text("Can not go there. Select an adjacent tile to your unit!");


       if e.type is MOUSEMOTION:
         mouse_pos = pygame.mouse.get_pos(); 
         mx, my = self.view.screen_to_tile(mouse_pos);
         px, py = self.map.player_pos;
         if (self.map.moves != 0 and (mx + 1 == px and my + 1 == py 
            or mx + 1 == px and my == py
            or mx + 1 == px and my - 1 == py
            or mx == px and my - 1 == py
            or mx == px and my + 1 == py
            or mx - 1 == px and my - 1 == py
            or mx - 1 == px and my == py
            or mx - 1 == px and my + 1 == py)):
           self.cursor.enable() 
         else:
           self.cursor.disable() 


       if e.type is MOUSEBUTTONDOWN and e.button == 1:
         mouse_pos = pygame.mouse.get_pos(); 
         x, y = self.view.screen_to_tile(mouse_pos);
         logging.info("tile %r %r   valid godo %r" % (x, y,self.map.can_goto((x, y)) ));

       x, y = self.map.player_pos;
       if e.type is KEYDOWN and not self.active_combat:
         if e.key == K_KP1: 
           y += 1;
           self.map.move_player_to((x,y));
         if e.key == K_KP2 or e.key == K_DOWN: 
           x += 1;
           y += 1;
           self.map.move_player_to((x,y));
         if e.key == K_KP3: 
           x += 1;
           self.map.move_player_to((x,y));
         if e.key == K_KP4 or e.key == K_LEFT: 
           x -= 1;
           y += 1;
           self.map.move_player_to((x,y));
         if e.key == K_KP6 or e.key == K_RIGHT:
           x += 1;
           y -= 1;
           self.map.move_player_to((x,y));
         if e.key == K_KP7: 
           x -= 1;
           self.map.move_player_to((x,y));
         if e.key == K_KP8 or e.key == K_UP: 
           x -= 1;
           y -= 1;
           self.map.move_player_to((x,y));
         if e.key == K_KP9: 
           y -= 1;
           self.map.move_player_to((x,y));

         if (e.key == K_RETURN or e.key == K_KP_ENTER) and self.highscores == 1:
           self.submit(None); 
           return;


         if e.key == K_RETURN or e.key == K_KP_ENTER:
           self.map.end_turn();

       self.app.event(e);
